package com.bbc.denadrive

import android.content.Context
import android.util.Log
import com.bbc.denadrive.apihandlers.DriveResponseBody
import com.bbc.denadrive.apihandlers.ResponsesHandler
import com.bbc.denadrive.apihandlers.makeDriveApiCall
import com.bbc.denadrive.oauth.AuthStatePersistence
import okhttp3.HttpUrl

object DriveRequester {

    fun makeNewRequest(
        context: Context,
        url: String,
        builderModifier: (HttpUrl.Builder) -> Unit,
        callback: (String?, Exception?) -> Unit
    ) {
        val authPersistence = AuthStatePersistence
        val authState = authPersistence.retrieveAuthState(context)
        makeDriveApiCall(context, authState, url, builderModifier, callback)
        authPersistence.putAuthState(context, authState)
    }

    fun populateTree(
        context: Context,
        tree: FolderTree
    ) {
        val myUrl = "https://www.googleapis.com/drive/v3/files?trashed=false&orderBy=folder"
        makeNewRequest(
            context = context,
            url = myUrl,
            builderModifier = { builder ->
                builder.addQueryParameter(
                    name = "q",
                    value = "'${tree.value.id}' in parents"
                )
            }
        ) { response, exception ->
            if (exception != null) {
                Log.e("RESPONSE EXCEPTION", "populateTree: ${exception.message}")
            } else {
                val responseBody = ResponsesHandler.newResponseArrived<DriveResponseBody>(response)
                if (responseBody != null) {
                    tree.addChildren(
                        responseBody.files.map { file ->
                            FolderTree(
                                value = file,
                                ownPath = "${tree.ownPath}/${file.id}",
                                parent = tree
                            )
                        }
                    )
                    tree.loaded = true
                    for (child in tree.getChildren()) {
                        if (!child.isLeaf()) populateTree(context, child)
                    }
                }
            }
        }
    }
}