/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive

import android.util.Log
import com.bbc.denadrive.apihandlers.MyFile
import java.util.stream.Collectors

class FolderTree(
    val value: MyFile,
    val ownPath: String = value.id,
    private val children: MutableList<FolderTree> = mutableListOf(),
    val parent: FolderTree? = null,
    var loaded: Boolean = false
) {

    fun isLeaf(): Boolean {
        return !value.isFolder()
    }

    fun addChild(file: MyFile) {
        children.add(FolderTree(file))
    }

    fun accChild(tree: FolderTree) {
        children.add(tree)
    }

    fun addChildren(trees: List<FolderTree>) {
        children.addAll(trees)
    }

    fun accChildren(files: List<MyFile>) {
        val trees = files.map { file -> FolderTree(file) }
        children.addAll(trees)
    }

    fun getChildren(): List<FolderTree> {
        return children
    }

    fun stepForward(folderId: String): FolderTree {
        if (folderId.isEmpty()) return this
        val didIFindIt =
            children.parallelStream().filter { tree -> tree.value.id == folderId }
                .collect(Collectors.toList())
        if (didIFindIt.isEmpty()) {
            throw PathNotFoundException("Folder with id $folderId not found")
        } else {
            return didIFindIt[0]
        }
    }

//    fun goToPath(path: String): FolderTree {
//        Log.e("INVOKED", "goToPath: $path", )
//        if (path.isEmpty()) return this
//        val target = path.substringBefore("/")
//        val idList =
//            children.parallelStream().filter { tree -> tree.value.id == target }
//                .collect(Collectors.toList())
//
//        if (idList.isEmpty()) {
//            Log.e("NOT FOUND", "goToPath: $children", )
//            throw PathNotFoundException("Folder with id $target not found")
//        } else {
//            return idList[0].goToPath(path.substringAfter("/"))
//        }
//    }

    override fun toString(): String {
        return "$value"
    }

}