/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive

import android.app.PendingIntent
import android.content.Intent
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material3.Button
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.painter.Painter
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.core.net.toUri
import com.bbc.denadrive.oauth.LoadingActivity
import com.bbc.denadrive.oauth.retrieveAuthServConf
import com.bbc.denadrive.ui.theme.DeNaDriveTheme
import net.openid.appauth.AuthorizationRequest
import net.openid.appauth.AuthorizationService
import net.openid.appauth.ResponseTypeValues

class HomeNotAuthActivity : ComponentActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        enableEdgeToEdge()
        setContent {
            DeNaDriveTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    val logo = painterResource(id = R.drawable.vecchiaicona_)
                    LogoWithCaptionAndButton(
                        logo = logo,
                        caption = "DeNa Drive",
                        buttonText = "Log in",
                        modifier = Modifier.padding(innerPadding)
                    ) { handleAuthStuff() }
                }
            }
        }
    }

    private fun handleAuthStuff() {
        val serviceConfig = retrieveAuthServConf()
        val authRequestBuilder = AuthorizationRequest.Builder(
            serviceConfig,
            "722393551256-pum20gbvb1es723kt3ek9lmtrdimr0os.apps.googleusercontent.com",
            ResponseTypeValues.CODE,
            "com.bbc.denadrive:/oauth2redirect".toUri()
        )
        val authRequest = authRequestBuilder
            .setScopes(
                listOf(
                    "https://www.googleapis.com/auth/drive",
                    "https://www.googleapis.com/auth/drive.apps"
                )
            )
            .build()

        val authService = AuthorizationService(this)
        authService.performAuthorizationRequest(
            authRequest,
            PendingIntent.getActivity(
                this, 0, Intent(this, LoadingActivity::class.java),
                PendingIntent.FLAG_MUTABLE
            ),
            PendingIntent.getActivity(
                this, 0, Intent(this, HomeNotAuthActivity::class.java),
                PendingIntent.FLAG_MUTABLE
            )
        )
    }
}

@Composable
fun Greeting4(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview4() {
    DeNaDriveTheme {
        Greeting4("Android")
    }
}

@Composable
fun LogoWithCaptionAndButton(
    logo: Painter, // This will be your SVG logo
    caption: String,
    buttonText: String,
    modifier: Modifier,
    onButtonClick: () -> Unit
) {
    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp),
        horizontalAlignment = Alignment.CenterHorizontally,
        verticalArrangement = Arrangement.Center
    ) {
        // Logo
        Image(
            painter = logo,
            contentDescription = "Logo",
            modifier = Modifier.size(200.dp) // Adjust size as needed
        )

        Spacer(modifier = Modifier.height(16.dp)) // Space between logo and caption

        // Caption
        Text(text = caption)

        Spacer(modifier = Modifier.height(16.dp)) // Space between caption and button

        // Button
        Button(onClick = onButtonClick) {
            Text(text = buttonText)
        }
    }
}
