/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.addCallback
import androidx.activity.compose.setContent
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.navigation.compose.rememberNavController
import com.bbc.denadrive.apihandlers.DriveResponseBody
import com.bbc.denadrive.apihandlers.MyFile
import com.bbc.denadrive.apihandlers.ResponsesHandler
import com.bbc.denadrive.apihandlers.ResponsesViewer
import com.bbc.denadrive.apihandlers.makeDriveApiCall
import com.bbc.denadrive.filehandling.FileDownloader
import com.bbc.denadrive.home.ScaffoldWithSidebar
import com.bbc.denadrive.oauth.AuthStatePersistence
import com.bbc.denadrive.ui.theme.DeNaDriveTheme
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationResponse
import okhttp3.HttpUrl

class MainActivity : ComponentActivity() {

    private lateinit var responsesViewer: ResponsesViewer
    private val exportSupportedFiles: List<String> = listOf(
        "document",
        "drawing",
        "presentation",
        "spreadsheet",
    )

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        responsesViewer = ResponsesViewer()

        DriveRequester.populateTree(this, responsesViewer.folderTree)

//        onBackPressedDispatcher.addCallback(this) {
//            if (currentPath != "root") goToFolder("", false)
//            else finish()
//        }

        onBackPressedDispatcher.addCallback(this) {
            if (responsesViewer.currentFolderTree.value.parent != null) {
                responsesViewer.currentFolderTree.value =
                    responsesViewer.currentFolderTree.value.parent!!
            } else {
                finish()
            }
        }
        val fileDownloader = FileDownloader(this)

//        goToFolder(parentFolder = "root", forward = true, isFirstRequest = true)

        setContent {
//            val navController = rememberNavController()
            ScaffoldWithSidebar(
                responsesViewer,
                { file ->
                    Toast.makeText(
                        this,
                        "You pressed the file ${file.name}",
                        Toast.LENGTH_SHORT
                    ).show()
                    if (file.mimeType.substringAfterLast(".") in exportSupportedFiles) {
                        fileDownloader.downloadFile(
                            "https://www.googleapis.com/drive/v3/files/${file.id}/export",
                            "application/pdf"
                        )
                    } else {
                        Toast.makeText(
                            this,
                            "This file has no default action. Download support coming soon",
                            Toast.LENGTH_SHORT
                        ).show()
                    }
                },
                { folder ->
                    responsesViewer.currentFolderTree.value =
                        responsesViewer.currentFolderTree.value.stepForward(folder.id)
//                    responsesViewer.currentFolderTree.value =
//                        responsesViewer.currentFolderTree.value.goToPath("$folder/")
//                    parent ->
//                    goToFolder(parent, true)
                }
            ) {
                responsesViewer.toggleUpdaters()
//                getApps()
            }
        }
    }


    private fun getApps() {
        val myUrl = "https://www.googleapis.com/drive/v3/apps"
        DriveRequester.makeNewRequest(this, myUrl, {  }) {
            response, exception ->
            if (exception != null) {
                Log.e("notGood", "getApps: $exception", )
            } else {
                Log.e("GOOD", "getApps: $response", )
            }
        }
    }

//    private fun askForFolder(folderId: String) {
//        val myUrl = "https://www.googleapis.com/drive/v3/files"
//        makeNewRequest(myUrl, { builder ->
//            builder.addQueryParameter("q", "'$folderId' in parents")
//        }) { response, exception ->
//            if (exception != null) {
//                Log.e("TAG", "askForFolder: Something went wrong")
//            } else {
//                // fai quel che ti pare
//            }
//        }
//    }
//
//    private fun goToFolder(
//        parentFolder: String, forward: Boolean, isFirstRequest: Boolean = false
//    ) {
//        responsesViewer.lastResponseBody.value = null
//        val actualFolder: String
//        if (!isFirstRequest) {
//            if (forward) {
//                currentPath += "/$parentFolder"
//                actualFolder = parentFolder
//            } else {
//                currentPath = currentPath.substringBeforeLast("/")
//                actualFolder =
//                    if (currentPath != "root") currentPath.substringAfterLast("/")
//                    else "root"
//            }
//        } else actualFolder = parentFolder
//
//        val myUrl = "https://www.googleapis.com/drive/v3/files?trashed=false&orderBy=folder"
//        makeNewRequest(myUrl, { builder ->
//            builder.addQueryParameter("q", "'$actualFolder' in parents")
//        }) { response, exception ->
//            if (exception != null) {
//                Log.e("ECCOLO", "${exception.cause}")
//                Log.e("ECCOLO", "${exception.message}")
//            } else {
//                Log.e("UOOOO", "$response")
//                responsesViewer.newResponseArrived(response)
//            }
//        }
//    }
//
//    private fun makeNewRequest(
//        url: String,
//        builderModifier: (HttpUrl.Builder) -> Unit,
//        callback: (String?, Exception?) -> Unit
//    ) {
//        val authPersistence = AuthStatePersistence
//        val authState = authPersistence.retrieveAuthState(this)
//        makeDriveApiCall(this, authState, url, builderModifier, callback)
//        authPersistence.putAuthState(this, authState)
//    }

//    override fun onNewIntent(intent: Intent) {
//        super.onNewIntent(intent)
//        val resp = AuthorizationResponse.fromIntent(intent)
//        val ex = AuthorizationException.fromIntent(intent)
//        if (resp != null) {
//            Toast.makeText(this, "YES", Toast.LENGTH_SHORT).show()
//        } else {
//            if (ex != null) {
//                Toast.makeText(this, "NOPE ${ex.error}", Toast.LENGTH_SHORT).show()
//            }
//        }
//
//        setContent {
////            val navController = rememberNavController()
////            ScaffoldWithSidebar(navController) {  }
//            Greeting("bah")
//        }
//    }
}

//@Composable
//fun Greeting(name: String, modifier: Modifier = Modifier) {
//    Text(
//        text = "Hello $name!",
//        modifier = modifier
//    )
//}
//
//@Preview(showBackground = true)
//@Composable
//fun GreetingPreview() {
//    DeNaDriveTheme {
//        Greeting("Android")
//    }
//}