/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive.apihandlers

import android.content.Context
import net.openid.appauth.AuthState
import net.openid.appauth.AuthorizationService
import okhttp3.Call
import okhttp3.Callback
import okhttp3.HttpUrl
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Response
import okhttp3.HttpUrl.Companion.toHttpUrl
import java.io.IOException

fun makeDriveApiCall(
    context: Context,
    authState: AuthState,
    url: String,
//    folderId: String,
    builderModifier: (HttpUrl.Builder) -> Unit,
    callback: (String?, Exception?) -> Unit
) {
    val authService = AuthorizationService(context)
    // _ = idToken
    authState.performActionWithFreshTokens(authService) { accessToken, _, ex ->
        authService.dispose()
        if (ex != null) {
            callback(null, ex)
            return@performActionWithFreshTokens
        }

        val urlBuilder = url.toHttpUrl().newBuilder()
        builderModifier(urlBuilder)
//        urlBuilder.addQueryParameter("q", "'$folderId' in parents")
        val updatedUrl = urlBuilder.build()

        val request = Request.Builder()
            .url(updatedUrl)
            .header("Authorization", "Bearer $accessToken")
            .build()

        OkHttpClient().newCall(request).enqueue(object : Callback {
            override fun onFailure(call: Call, e: IOException) {
                callback(null, e)
            }

            override fun onResponse(call: Call, response: Response) {
                val responseBody = response.body?.string()
                callback(responseBody, null)
            }
        })
    }
}
