package com.bbc.denadrive.filehandling

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.util.Log
import androidx.core.content.FileProvider
import com.bbc.denadrive.oauth.AuthStatePersistence
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import okhttp3.HttpUrl.Companion.toHttpUrl
import java.io.File
import java.io.FileOutputStream
import java.net.HttpURLConnection
import java.net.URL

class FileDownloader(private val context: Context) {

    fun downloadFile(fileUrl: String, mimeType: String) {
        CoroutineScope(Dispatchers.IO).launch {
            val tempFile = downloadIt(fileUrl, mimeType)
            withContext(Dispatchers.Main) {
                tempFile?.let {
                    openFile(it)
                }
            }
        }
    }

    private fun downloadIt(fileUrl: String, mimeType: String): File? {
        var tempFile: File? = null
        val authState = AuthStatePersistence.retrieveAuthState(context)

        try {
            tempFile = File.createTempFile("tempFile", ".pdf", context.cacheDir)
//            val builder = fileUrl.toHttpUrl().newBuilder()
//            builder.addQueryParameter("mimeType", mimeType)
//            val urlConnection = builder.build().toUrl().openConnection() as HttpURLConnection
            val urlConnection = URL("$fileUrl?mimeType=application/pdf").openConnection() as HttpURLConnection
            urlConnection.setRequestProperty("Authorization", "Bearer ${authState.accessToken}")
            urlConnection.connect()

//            val inputStream1 = urlConnection.inputStream
//            val buffer1 = ByteArray(4)
//            inputStream1.read(buffer1)
//            val header = String(buffer1)
//            if (!header.startsWith("%PDF")) {
//                Log.e("FileDownloader", "Downloaded file is not a valid PDF")
//            }

            if (urlConnection.responseCode == HttpURLConnection.HTTP_OK) {
                Log.e("MA", "downloadIt: andato", )
                val inputStream = urlConnection.inputStream
                val outputStream = FileOutputStream(tempFile)

                val buffer = ByteArray(1024)
                var bytesRead: Int
                while (inputStream.read(buffer).also { bytesRead = it } != -1) {
                    outputStream.write(buffer, 0, bytesRead)
                }

                outputStream.close()
                inputStream.close()
                urlConnection.disconnect()
            } else {
                Log.e("FileDownloader", "Error: ${urlConnection.responseCode} - ${urlConnection.responseMessage}")
            }

        } catch (e: Exception) {
            Log.e("FileDownloader", "downloadIt: download failed", e)
        }

        return tempFile
    }


    private fun openFile(file: File) {
        val uri: Uri = FileProvider.getUriForFile(context, "${context.packageName}.fileprovider", file)
        val intent = Intent(Intent.ACTION_VIEW).apply {
            setDataAndType(uri, "application/pdf")
            addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
        }
        context.startActivity(intent)
    }

}