/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive.home

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKey
import com.bbc.denadrive.HomeNotAuthActivity
import com.bbc.denadrive.MainActivity
import com.bbc.denadrive.home.ui.theme.DeNaDriveTheme
import com.bbc.denadrive.oauth.AuthStatePersistence
import net.openid.appauth.AuthState

class NeverLoggedInLogic : ComponentActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        val authPersistence = AuthStatePersistence

        val intent: Intent
        if (!authPersistence.isAuthStateSet(this)) {
            intent = Intent(this, HomeNotAuthActivity::class.java)
        } else {
            val authState = authPersistence.retrieveAuthState(this)
            intent = if (authState.isAuthorized) {
                Intent(this, MainActivity::class.java)
            }
            else {
                Toast.makeText(this, "Session expired, please log in again",
                    Toast.LENGTH_SHORT).show()
                Intent(this, HomeNotAuthActivity::class.java)
            }
        }
        startActivity(intent)

        enableEdgeToEdge()
        setContent {
            DeNaDriveTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    Greeting5(
                        name = "Android",
                        modifier = Modifier.padding(innerPadding)
                    )
                }
            }
        }
        finish()
    }
}

@Composable
fun Greeting5(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview5() {
    DeNaDriveTheme {
        Greeting5("Android")
    }
}