/*
This file is part of DeNa Drive.

DeNa Drive is free software: you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

DeNa Drive is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with DeNa Drive.
If not, see <https://www.gnu.org/licenses/>.
*/

package com.bbc.denadrive.oauth

import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material3.CircularProgressIndicator
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.security.crypto.EncryptedSharedPreferences
import androidx.security.crypto.MasterKey
import com.bbc.denadrive.HomeNotAuthActivity
import com.bbc.denadrive.MainActivity
import com.bbc.denadrive.oauth.ui.theme.DeNaDriveTheme
import net.openid.appauth.AuthState
import net.openid.appauth.AuthorizationException
import net.openid.appauth.AuthorizationResponse
import net.openid.appauth.AuthorizationService

class LoadingActivity : ComponentActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        enableEdgeToEdge()
        setContent {
            DeNaDriveTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    LoadingScreen(modifier = Modifier.padding(innerPadding))
                }
            }
        }

        val authPersistence = AuthStatePersistence

        val authState = authPersistence.retrieveAuthState(this)

        val response = AuthorizationResponse.fromIntent(intent)
        val exception = AuthorizationException.fromIntent(intent)
        if (response != null) {
            Toast.makeText(this, "Done!", Toast.LENGTH_SHORT).show()
            authPersistence.update(this, authState, response, exception)
            Log.e("MainActivity", "authState: ${authState.jsonSerializeString()}", )
        } else {
            Toast.makeText(this, "Not done :( ${exception?.error}", Toast.LENGTH_SHORT).show()
        }

        val authService = AuthorizationService(this)
        response?.createTokenExchangeRequest()?.let {
            authService.performTokenRequest(
                it
            ) { resp, ex ->
                if (resp != null) {
                    authPersistence.update(this, authState, resp, ex)
                    val intent = Intent(this, MainActivity::class.java)
                    startActivity(intent)
                } else {
                    val intent = Intent(this, HomeNotAuthActivity::class.java)
                    startActivity(intent)
                }
            }
        }

    }
}

@Composable
fun LoadingScreen(
    modifier: Modifier = Modifier
) {
    // State to control the rotation animation
    val rotation = remember { mutableFloatStateOf(0f) }

    // Animate the rotation
    LaunchedEffect(Unit) {
        while (true) {
            rotation.floatValue += 10f
            if (rotation.floatValue >= 360f) rotation.floatValue = 0f
            kotlinx.coroutines.delay(16) // Approximately 60 FPS
        }
    }

    Box(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp),
        contentAlignment = Alignment.Center
    ) {
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            verticalArrangement = Arrangement.Center
        ) {
            // Rotating Circular Progress Indicator
            CircularProgressIndicator(
                modifier = Modifier
                    .size(64.dp)
                    .graphicsLayer(rotationZ = rotation.floatValue),
                color = Color.Blue
            )
            Spacer(modifier = Modifier.height(16.dp))
            // Loading text
            Text(
                text = "Loading",
                fontSize = 20.sp,
                fontWeight = FontWeight.Bold,
                color = Color.Black
            )
        }
    }
}

@Composable
fun Greeting6(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview6() {
    DeNaDriveTheme {
        Greeting6("Android")
    }
}